--PLACE LEADING ZEROS
local function lead_zeros(num)
	local ret = num
	if num < 10 then
		ret = "0"..num
	end
	return ret
end

--OUTPUT
local function output(span, label, mark, leading)
	if not leading then leading = false end
	if(span.used and label ~= nil) then
		if(span.template ~= "") then
			if(leading) then
				label:setText(span.template:gsub(mark, lead_zeros(span.value)))
			else
				label:setText(span.template:gsub(mark, span.value));
			end
		else
			if(leading)then
				label:setText(lead_zeros(span.value));
			else
				label:setText(span.value);
			end
		end
	end
end

--IF COUNTDOWN ENDED
local function is_ended(abr)
	local is_zero = true
	for i in pairs(abr) do
		if(abr[i].used and abr[i].value < 0) then
			return true
		end
		if(abr[i].value > 0) then
			is_zero = false;
		end
	end
	return is_zero
end

Countdown = gideros.class(Sprite)
function Countdown:init(config)
	self.timer = Timer.new(1000)
	self.timer:addEventListener(Event.TIMER, function()
		self:update()
	end)
	
	self.conf = {
		time = 0,
		hour = 0,
		min = 0,
		sec = 0,
		--FOR TIME LABELS
		label_hour = nil,
		label_min = nil,
		label_sec = nil,
		
		--END LABEL
		label_end = nil,
		
		--SETTINGS
		hide_zeros = true,
		leading_zeros = true,
		onend = nil,
		onstep = nil
	}
	
	if config then
		for key,value in pairs(config) do
			self.conf[key] = value
		end
	end

	self.offset = false
	self.abr = {
		h = {name = "hour", used = false, value = 0, template=""},
		i = {name = "min", used = false, value = 0, template=""},
		s = {name = "sec", used = false, value = 0, template=""}
	}
	
	for i in pairs(self.abr) do
		if(self.conf["label_"..self.abr[i].name] ~= nil) then
			self:addChild(self.conf["label_"..self.abr[i].name])
			self.abr[i].used = true
			if(self.conf["label_"..self.abr[i].name]:getText():find("{"..i.."}") ~= nil) then
				self.abr[i].template = self.conf["label_"..self.abr[i].name]:getText()
				self.conf["label_"..self.abr[i].name]:setText("")
			end
		end
	end
	if self.conf.label_end ~= nil then
		self.conf.label_end:setVisible(false)
		self:addChild(self.conf.label_end)
	end
	if(self.conf.time > 0) then
		self.conf.sec = os.difftime( self.conf.time, os.time() )
	end

	self:normalize()
	self:recalc()
	self:update()
	self:start()
end

--START COUNTDOWN
function Countdown:start()
	if not self.started then
		self.started = true
		self.timer:start()
	end
end

--STOP COUNTDOWN
function Countdown:stop()
	if self.started then
		self.started = false
		self.timer:stop()
	end
end

--UPDATE COUNTDOWN ELEMENTS
function Countdown:update()
	self.conf.sec = self.conf.sec - 1
	if(self.conf.sec == -1) then
		self.conf.min = self.conf.min - 1
		self.conf.sec = 59
	end
	if(self.conf.min == -1) then
		self.conf.hour = self.conf.hour - 1
		self.conf.min = 59
	end
	self:recalc();
	if(self.conf.onstep) then
		self.conf.onstep(self:getSeconds());
	end
	output(self.abr.i, self.conf.label_min, "{i}", self.conf.leading_zeros);
	output(self.abr.s, self.conf.label_sec, "{s}", self.conf.leading_zeros);
	if(self.conf.hide_zeros) then
		local hide = true;
		if(self.abr.i.used) then
			if(self.abr.i.value <= 0 and hide) then
				self.conf.label_min:setVisible(true);
			else
				hide = false;
			end
		end
		if(self.abr.s.used) then
			if(self.abr.s.value <= 0 and hide) then
				self.conf.label_sec:setVisible(false);
			else
				hide = false;
			end
		end
	end
	if(is_ended(self.abr)) then
		for i in pairs(self.abr) do
			if(self.abr[i].used) then
				self.conf["label_"..self.abr[i].name]:setVisible(true);
			end
		end
		if(self.conf.label_end ~= nil) then
			self.conf.label_end:setVisible(true);
		end
		self:stop()
		if(self.conf.onend) then
			self.conf.onend();
		end
	end
end

--SECONDS LEFT
function Countdown:getSeconds()
	local arr = {}
	for i in pairs(self.abr) do
		arr[i] = {};
		arr[i].value = self.conf[self.abr[i].name]
	end
	arr.i.value = arr.i.value + (arr.h.value*60)
	arr.s.value = arr.s.value + (arr.i.value*60)
	return arr.s.value
end

function Countdown:normalize()
	self.conf.min = self.conf.min + math.floor(self.conf.sec/60);
	self.conf.sec = self.conf.sec % 60;
	
	self.conf.hour = self.conf.hour + math.floor(self.conf.min/60);
	self.conf.min = self.conf.min % 60;
end

function Countdown:recalc()
	for i, value in pairs(self.abr) do
		self.abr[i].value = self.conf[self.abr[i].name];
	end
	if(not self.abr.i.used) then
		self.abr.s.value = self.abr.s.value + (self.abr.i.value*60);
		self.abr.i.value = 0;
	end
	if(not self.abr.s.used) then
		self.abr.s.value = 0;
	end
end